from __future__ import annotations

import json
from typing import Any

from .resources import resources


def get_schema(tool_name: str = "check-sdist") -> dict[str, Any]:
    "Get the stored complete schema for check-sdist settings."
    assert tool_name == "check-sdist", "Only check-sdist is supported."

    with resources.joinpath("check-sdist.schema.json").open(encoding="utf-8") as f:
        return json.load(f)  # type: ignore[no-any-return]



#!/usr/bin/env python3

# /// script
# dependencies = ["pyyaml"]
# ///
from __future__ import annotations

import json

import yaml

starter = """
$schema: http://json-schema.org/draft-07/schema#
$id: https://json.schemastore.org/partial-check-sdist.json
additionalProperties: false
description: check-sdist's settings.
type: object
properties:
  sdist-only:
    description: Files that only are in the SDist. Gitignore style lines.
    type: array
    items:
      type: string
  git-only:
    description: Files that are only in Git (or whatever mode you use). Gitignore style lines.
    type: array
    items:
      type: string
  default-ignore:
    description: Ignore some common files
    default: true
    type: boolean
  recurse-submodules:
    description: Look in all submodules too.
    default: true
    type: boolean
  mode:
    description: What to use as baseline
    default: git
    enum:
      - git
      - all
  build-backend:
    description: What to expect as build-backend, in order to look for exclude lists
    default: auto
    enum:
      - auto
      - none
      - flit_core.buildapi
      - hatchling.build
      - scikit_build_core.build
      - pdm.backend.api
      - poetry.core.masonry.api
      - maturin
"""

schema = yaml.safe_load(starter)
print(json.dumps(schema, indent=2))



"""
Copyright (c) 2023 Henry Schreiner. All rights reserved.

check-sdist: Check the contents of an SDist vs. git
"""

from __future__ import annotations

__version__ = "1.4.0"

__all__ = ["__version__"]



from __future__ import annotations

import subprocess
import sys
from pathlib import Path
from typing import Literal

import pytest

from check_sdist.__main__ import compare
from check_sdist._compat import tomllib
from check_sdist.sdist import get_uv

DIR = Path(__file__).parent.resolve()

with DIR.joinpath("downstream.toml").open("rb") as f:
    packages = tomllib.load(f)["packages"]


@pytest.mark.parametrize(
    "installer",
    [
        pytest.param(
            "uv", marks=pytest.mark.skipif(get_uv() is None, reason="uv not found")
        ),
        "pip",
    ],
)
@pytest.mark.parametrize(
    ("repo", "ref", "fail"), [(x["repo"], x["ref"], x.get("fail", 0)) for x in packages]
)
def test_packages(
    repo: str,
    ref: str,
    fail: int,
    tmp_path: Path,
    monkeypatch: pytest.MonkeyPatch,
    installer: Literal["uv", "pip"],
):
    if repo.endswith("scikit-build") and sys.platform.startswith("win32"):
        pytest.skip(reason="Path too long on Windows (0.18)")

    monkeypatch.chdir(tmp_path)
    cmd = [
        "git",
        "clone",
        f"https://github.com/{repo}",
        "--depth=1",
        "--branch",
        ref,
        "--recurse-submodules",
    ]
    subprocess.run(cmd, check=True)
    package_path = tmp_path / repo.split("/")[1]
    assert compare(package_path, isolated=True, installer=installer) == fail



from __future__ import annotations

import inspect
import os
import shutil
import subprocess
from pathlib import Path

import pytest

from check_sdist.__main__ import compare
from check_sdist.inject import inject_junk_files

DIR = Path(__file__).parent


@pytest.fixture
def self_repo(tmp_path: Path) -> Path:
    repo = tmp_path / "repo"
    shutil.copytree(
        DIR.parent, repo, ignore=shutil.ignore_patterns("*venv*", "*cache*", "dist")
    )
    return repo


def get_all_files(path: Path) -> frozenset[str]:
    with os.scandir(path) as it:
        return frozenset(it.name for it in it if not it.name.startswith(".coverage"))


def test_self_dir():
    start = get_all_files(DIR.parent)
    assert compare(DIR.parent, isolated=True) == 0
    end = get_all_files(DIR.parent)
    assert start == end


def test_self_dir_injected(self_repo: Path, monkeypatch: pytest.MonkeyPatch):
    monkeypatch.chdir(self_repo)
    start = get_all_files(Path())
    with inject_junk_files(Path()):
        assert compare(Path(), isolated=True) == 0
    end = get_all_files(Path())
    assert start == end


@pytest.fixture
def git_dir(tmp_path: Path, monkeypatch: pytest.MonkeyPatch):
    monkeypatch.chdir(tmp_path)
    subprocess.run(["git", "init"], check=True)
    subprocess.run(["git", "config", "user.email", "ci@example.com"], check=True)
    subprocess.run(["git", "config", "user.name", "CI"], check=True)
    Path(".gitignore").write_text(
        inspect.cleandoc("""
        some-ignored-file.txt
    """)
    )
    Path("example").mkdir()
    Path("example/__init__.py").touch()
    Path("example/notme.py").touch()
    Path("ignore-me.txt").touch()
    Path("not-ignored.txt").touch()
    Path("some-file").touch()
    Path("some-dir").mkdir()
    Path("some-dir/notme.py").touch()
    Path("some-dir/more-dir").mkdir()
    Path("some-dir/more-dir/notme.py").touch()
    Path("some-ignored-file.txt").touch()

    return tmp_path


@pytest.mark.usefixtures("git_dir")
@pytest.mark.parametrize("backend", ["auto", "none"])
def test_hatchling(backend: str):
    Path("pyproject.toml").write_text(
        inspect.cleandoc(f"""
            [build-system]
            requires = ["hatchling"]
            build-backend = "hatchling.build"

            [project]
            name = "example"
            version = "0.1.0"

            [tool.hatch]
            build.targets.sdist.exclude = ["ignore*", "some-file", "**/notme.py"]

            [tool.check-sdist]
            build-backend = "{backend}"
        """)
    )
    subprocess.run(["git", "add", "."], check=True)
    subprocess.run(["git", "commit", "-m", "Initial commit"], check=True)
    assert compare(Path(), isolated=True, verbose=True) == (
        0 if backend == "auto" else 2
    )


@pytest.mark.usefixtures("git_dir")
@pytest.mark.parametrize("backend", ["auto", "none"])
def test_flit_core(backend: str):
    Path("pyproject.toml").write_text(
        inspect.cleandoc(f"""
            [build-system]
            requires = ["flit-core"]
            build-backend = "flit_core.buildapi"

            [project]
            name = "example"
            version = "0.1.0"
            description = "A test package"

            [tool.flit.sdist]
            include = ["not-ignored.txt", ".gitignore"]
            exclude = ["ignore*", "some-file", "**/notme.py"]

            [tool.check-sdist]
            build-backend = "{backend}"
        """)
    )
    subprocess.run(["git", "add", "."], check=True)
    subprocess.run(["git", "commit", "-m", "Initial commit"], check=True)
    assert compare(Path(), isolated=True, verbose=True) == (
        0 if backend == "auto" else 2
    )


@pytest.mark.usefixtures("git_dir")
@pytest.mark.parametrize("backend", ["auto", "none"])
def test_scikit_build_core(backend: str):
    Path("pyproject.toml").write_text(
        inspect.cleandoc(f"""
            [build-system]
            requires = ["scikit-build-core"]
            build-backend = "scikit_build_core.build"

            [project]
            name = "example"
            version = "0.1.0"

            [tool.scikit-build]
            sdist.exclude = ["ignore*", "some-file", "**/notme.py"]

            [tool.check-sdist]
            build-backend = "{backend}"
        """)
    )
    subprocess.run(["git", "add", "."], check=True)
    subprocess.run(["git", "commit", "-m", "Initial commit"], check=True)
    assert compare(Path(), isolated=True, verbose=True) == (
        0 if backend == "auto" else 2
    )


@pytest.mark.usefixtures("git_dir")
@pytest.mark.parametrize("backend", ["auto", "none"])
def test_pdm_backend(backend: str):
    Path("pyproject.toml").write_text(
        inspect.cleandoc(f"""
            [build-system]
            requires = ["pdm-backend"]
            build-backend = "pdm.backend"

            [project]
            name = "example"
            version = "0.1.0"

            [tool.pdm]
            build.source-includes = [".gitignore", "not-ignored.txt"]
            build.excludes = ["ignore*", "some-file", "**/notme.py"]

            [tool.check-sdist]
            build-backend = "{backend}"
        """)
    )
    subprocess.run(["git", "add", "."], check=True)
    subprocess.run(["git", "commit", "-m", "Initial commit"], check=True)
    assert compare(Path(), isolated=True, verbose=True) == (
        0 if backend == "auto" else 2
    )


@pytest.mark.usefixtures("git_dir")
@pytest.mark.parametrize("backend", ["auto", "none"])
def test_maturin(backend: str):
    Path("pyproject.toml").write_text(
        inspect.cleandoc(f"""
            [build-system]
            requires = ["maturin~=1.0"]
            build-backend = "maturin"

            [project]
            name = "example"
            version = "0.1.0"

            [tool.maturin]
            features = ["pyo3/extension-module"]
            exclude = ["ignore*", "some-file", "**/notme.py", "Cargo.lock"]

            [tool.check-sdist]
            build-backend = "{backend}"
        """)
    )
    Path("Cargo.toml").write_text(
        inspect.cleandoc("""
            [package]
            name = "example"
            version = "0.1.0"
            edition = "2021"

            [dependencies.pyo3]
            version = "0.22.4"
        """)
    )
    Path("src").mkdir()
    Path("src/lib.rs").write_text("")
    subprocess.run(["git", "add", "."], check=True)
    subprocess.run(["git", "commit", "-m", "Initial commit"], check=True)
    assert compare(Path(), isolated=True, verbose=True) == (
        0 if backend == "auto" else 2
    )


@pytest.mark.usefixtures("git_dir")
@pytest.mark.parametrize("backend", ["auto", "none"])
def test_poetry_core(backend: str):
    Path("pyproject.toml").write_text(
        inspect.cleandoc(f"""
            [build-system]
            requires = ["poetry-core"]
            build-backend = "poetry.core.masonry.api"

            [tool.poetry]
            name = "example"
            version = "0.1.0"
            authors = []
            description = "A test package."
            include = [".gitignore", "not-ignored.txt"]
            exclude = ["ignore*", {{path="some-file", format=["sdist"]}}, "**/notme.py"]

            [tool.check-sdist]
            build-backend = "{backend}"
        """)
    )
    subprocess.run(["git", "add", "."], check=True)
    subprocess.run(["git", "commit", "-m", "Initial commit"], check=True)
    assert compare(Path(), isolated=True, verbose=True) == (
        0 if backend == "auto" else 2
    )


@pytest.mark.usefixtures("git_dir")
def test_unknown_backend():
    Path("pyproject.toml").write_text(
        inspect.cleandoc("""
            [build-system]
            requires = ["flit-core"]
            build-backend = "flit_core.buildapi"

            [project]
            name = "example"
            version = "0.1.0"
            description = "A test package"

            [tool.check-sdist]
            build-backend = "unknown"
        """)
    )

    subprocess.run(["git", "add", "."], check=True)
    subprocess.run(["git", "commit", "-m", "Initial commit"], check=True)
    with pytest.raises(ValueError, match="Unknown backend: unknown"):
        compare(Path(), isolated=True, verbose=True)



from __future__ import annotations

import pytest
import validate_pyproject.api


def test_validate_pyproject_defaults():
    example = {
        "tool": {
            "check-sdist": {
                "sdist-only": [],
                "git-only": [],
                "default-ignore": True,
                "recurse-submodules": True,
            }
        }
    }
    validator = validate_pyproject.api.Validator()
    assert validator(example) is not None


def test_validate_pyproject_extra_key():
    example: dict[str, object] = {
        "tool": {
            "check-sdist": {
                "sdists-only": [],
            }
        }
    }
    validator = validate_pyproject.api.Validator()
    with pytest.raises(validate_pyproject.error_reporting.ValidationError):
        validator(example)


def test_validate_pyproject_invalid_value():
    example = {
        "tool": {
            "check-sdist": {
                "sdist-only": [1],
            }
        }
    }
    validator = validate_pyproject.api.Validator()
    with pytest.raises(validate_pyproject.error_reporting.ValidationError):
        validator(example)



from __future__ import annotations

import argparse
import contextlib
from pathlib import Path
from typing import TYPE_CHECKING, Literal

import pathspec

from . import __version__
from ._compat import tomllib
from .backends import backend_ignored_patterns
from .git import git_files
from .inject import inject_junk_files
from .resources import resources
from .sdist import default_sdist_ignore, get_uv, sdist_files

if TYPE_CHECKING:
    from collections.abc import Sequence


def select_installer(
    installer: Literal["pip", "uv", "uv|pip"],
) -> Literal["uv", "pip"]:
    """
    Select uv, pip, or uv if available, then pip ("uv|pip"). Returns uv, pip,
    or throws an error if uv was required and not available.
    """
    if "uv" in installer:
        if get_uv() is not None:
            return "uv"

        if installer == "uv":
            msg = "Can't find uv"
            raise ImportError(msg)

    return "pip"


def compare(
    source_dir: Path,
    *,
    isolated: bool,
    verbose: bool = False,
    installer: Literal["uv", "pip", "uv|pip"] = "uv|pip",
) -> int:
    """
    Compare the files in the SDist with the files tracked by git.

    Takes the source directory and a flag indicating whether the SDist should
    be built in an isolated environment.

    Return 0 if they match, 1 if the SDist has files that are not tracked by
    git, 2 if the SDist is missing files that are tracked by git, and 3 if both
    conditions are true.
    """

    installer = select_installer(installer)

    pyproject = {}
    config = {}
    pyproject_toml = source_dir.joinpath("pyproject.toml")
    with contextlib.suppress(FileNotFoundError), pyproject_toml.open("rb") as f:
        pyproject = tomllib.load(f)
        config = pyproject.get("tool", {}).get("check-sdist", {})

    sdist_only_patterns = set(config.get("sdist-only", []))
    git_only_patterns = set(config.get("git-only", []))
    default_ignore = config.get("default-ignore", True)
    recurse_submodules = config.get("recurse-submodules", True)
    mode = config.get("mode", "git")
    backend = config.get("build-backend", "auto")

    sdist = sdist_files(source_dir, isolated=isolated, installer=installer) - {
        "PKG-INFO"
    }
    if mode == "git":
        git = git_files(source_dir, recurse_submodules=recurse_submodules)
    elif mode == "all":
        git = frozenset(
            str(p.relative_to(source_dir)) for p in source_dir.rglob("*") if p.is_file()
        )
    else:
        msg = "Only 'all' and 'git' supported for 'mode'"
        raise ValueError(msg)

    if default_ignore:
        with resources.joinpath("default-ignore.txt").open("r", encoding="utf-8") as f:
            git_only_patterns.update(f.read().splitlines())
        sdist_only_patterns.update(default_sdist_ignore(pyproject))

    sdist_spec = pathspec.GitIgnoreSpec.from_lines(sdist_only_patterns)
    git_spec = pathspec.GitIgnoreSpec.from_lines(git_only_patterns)

    sdist_only = frozenset(p for p in sdist - git if not sdist_spec.match_file(p))
    git_only = frozenset(p for p in git - sdist if not git_spec.match_file(p))

    git_only = backend_ignored_patterns(backend, pyproject, git_only)

    if verbose:
        print("SDist contents:")
        print(*(f"  {x}" for x in sorted(sdist)), sep="\n")
        print()

    if sdist_only or git_only:
        print("SDist does not match git")
        print()
        print("SDist only:")
        print(*(f"  {x}" for x in sorted(sdist_only)), sep="\n")
        print()
        print("Git only:")
        print(*(f"  {x}" for x in sorted(git_only)), sep="\n")
        print()
        return bool(sdist_only) + 2 * bool(git_only)

    print("SDist matches git")
    return 0


def main(sys_args: Sequence[str] | None = None, /) -> None:
    """Parse the command line arguments and call compare()."""

    parser = argparse.ArgumentParser(
        prog=None if sys_args is None else "check-sdist",
        allow_abbrev=False,
    )
    parser.add_argument(
        "--version", action="version", version=f"%(prog)s {__version__}"
    )
    parser.add_argument(
        "--source-dir",
        type=Path,
        default=Path.cwd(),
        help="The source directory to check (default: current directory)",
    )
    parser.add_argument(
        "--no-isolation",
        action="store_true",
        help="Do not build the SDist in an isolated environment",
    )
    parser.add_argument(
        "--inject-junk",
        action="store_true",
        help="Temporarily inject common junk files into the source directory",
    )
    parser.add_argument(
        "-v",
        "--verbose",
        action="store_true",
        help="Print out SDist contents too",
    )
    parser.add_argument(
        "--installer",
        choices={"uv", "pip", "uv|pip"},
        default="uv|pip",
        help="Tool to use when installing packages for making the SDist",
    )
    args = parser.parse_args(sys_args)

    with contextlib.ExitStack() as stack:
        if args.inject_junk:
            stack.enter_context(inject_junk_files(args.source_dir))

    raise SystemExit(
        compare(
            args.source_dir,
            isolated=not args.no_isolation,
            verbose=args.verbose,
            installer=args.installer,
        )
    )


if __name__ == "__main__":
    main()



from __future__ import annotations

from typing import TYPE_CHECKING

from check_sdist.inject import inject_files, inject_junk_files

if TYPE_CHECKING:
    from pathlib import Path


def test_inject_files(tmp_path: Path):
    with inject_files(tmp_path, ["a", "b", "c/", "d/e", "f/g/"]):
        assert {x.name for x in tmp_path.iterdir()} == {"a", "b", "c", "d", "f"}
        assert tmp_path.joinpath("a").is_file()
        assert tmp_path.joinpath("c").is_dir()
        assert tmp_path.joinpath("d/e").is_file()
        assert tmp_path.joinpath("f/g").is_dir()
    assert not set(tmp_path.iterdir())


def test_inject_junk_files(tmp_path: Path):
    with inject_junk_files(tmp_path):
        assert tmp_path.joinpath(".tox").is_dir()
        assert tmp_path.joinpath("tests").is_dir()
        assert tmp_path.joinpath("__pycache__").is_dir()
        assert tmp_path.joinpath("tests/__pycache__").is_dir()
    assert not set(tmp_path.iterdir())


def test_inject_junk_files_keep_dirs(tmp_path: Path):
    tmp_path.joinpath("tests/simple").mkdir(parents=True)
    with inject_junk_files(tmp_path):
        assert tmp_path.joinpath(".tox").is_dir()
        assert tmp_path.joinpath("tests/simple").is_dir()
        assert tmp_path.joinpath("tests/__pycache__").is_dir()
    assert {x.name for x in tmp_path.iterdir()} == {"tests"}
    assert {x.name for x in tmp_path.joinpath("tests").iterdir()} == {"simple"}
    assert not set(tmp_path.joinpath("tests/simple").iterdir())



from __future__ import annotations

import contextlib
import sys
from typing import TYPE_CHECKING

from .resources import resources

if TYPE_CHECKING:
    from collections.abc import Generator, Sequence
    from pathlib import Path

__all__ = ["inject_files", "inject_junk_files"]


@contextlib.contextmanager
def inject_files(source_dir: Path, files: Sequence[str]) -> Generator[None, None, None]:
    """This context manager will inject files into a directory, cleaned afterwards."""
    injected_files = []
    injected_directories = []

    for file in files:
        added_file = source_dir / (f"{file}/junk.py" if file.endswith("/") else file)
        if added_file.exists():
            continue

        # Record the missing directories
        parent_folder = added_file.parent
        while not parent_folder.is_dir():
            injected_directories.append(parent_folder)
            parent_folder = parent_folder.parent

        # Create the missing parent directories
        if not added_file.parent.exists():
            added_file.parent.mkdir(parents=True)

        injected_files.append(added_file)
        added_file.touch(exist_ok=False)
    try:
        yield
    finally:
        for injected in injected_files:
            injected.unlink()
        for injected in sorted(
            injected_directories, key=lambda x: len(str(x)), reverse=True
        ):
            injected.rmdir()


@contextlib.contextmanager
def inject_junk_files(source_dir: Path) -> Generator[None, None, None]:
    """This context manager will inject common junk files into a directory, cleaned afterwards."""
    with resources.joinpath("junk-paths.txt").open("r", encoding="utf-8") as f:
        junk_files = [ln.strip() for ln in f]
    # Windows does not allow question marks in filenames
    if sys.platform.startswith("win"):
        junk_files = [ln for ln in junk_files if "?" not in ln]
    with inject_files(source_dir, junk_files):
        yield



from __future__ import annotations

import shutil
import subprocess
import sys
import tarfile
import tempfile
from pathlib import Path
from typing import TYPE_CHECKING, Any, Literal

if TYPE_CHECKING:
    from collections.abc import Iterator, Mapping

__all__ = ["default_sdist_ignore", "get_uv", "sdist_files"]


def get_uv() -> str | None:
    """
    Find uv and return the arguments to use it.
    """
    try:
        # pylint: disable-next=import-outside-toplevel
        import uv  # noqa: PLC0415

        return uv.find_uv_bin()
    except ModuleNotFoundError:
        return shutil.which("uv")


def sdist_files(
    source_dir: Path, *, isolated: bool, installer: Literal["uv", "pip"]
) -> frozenset[str]:
    """Return the files that would be (are) placed in the SDist."""

    with tempfile.TemporaryDirectory() as outdir:
        if installer == "pip":
            cmd = [
                sys.executable,
                "-m",
                "build",
                "--sdist",
                "--outdir",
                outdir,
                f"--installer={installer}" if isolated else "--no-isolation",
            ]
        else:
            uv = get_uv()
            assert uv is not None, "uv must be found to reach this point!"
            cmd = [
                uv,
                "build",
                "--sdist",
                "--python",
                sys.executable,
                "--out-dir",
                outdir,
            ]

        subprocess.run(cmd, check=True, cwd=source_dir)

        (outpath,) = Path(outdir).glob("*.tar.gz")

        with tarfile.open(outpath) as tar:
            prefixes = {n.split("/", maxsplit=1)[0] for n in tar.getnames()}
            if len(prefixes) != 1:
                msg = f"malformed SDist, contains multiple packages {prefixes}"
                raise AssertionError(msg)
            return frozenset(
                t.name.split("/", maxsplit=1)[1]
                for t in tar.getmembers()
                if t.isfile() or t.issym()
            )


def default_sdist_ignore(pyproject: Mapping[str, Any]) -> Iterator[str]:
    """Return the default ignore patterns for SDists."""
    yield "*.dist-info"
    build_backend = pyproject.get("build-system", {}).get(
        "build-backend", "setuptools.build_meta.__legacy__"
    )
    if build_backend in {
        "setuptools.build_meta",
        "setuptools.build_meta.__legacy__",
    }:
        yield "*.egg-info"
        yield "setup.cfg"

    # Look for some common autogenerated version files
    # [tool.setuptools_scm]
    # version-file = "_version.py"
    setuptools_scm = (
        pyproject.get("tool", {}).get("setuptools_scm", {}).get("version_file", None)
    )
    if setuptools_scm is not None:
        yield setuptools_scm

    # [tool.hatch.build.hooks.vcs]
    # version-file = "_version.py"
    hatch_vcs = (
        pyproject.get("tool", {})
        .get("hatch", {})
        .get("build", {})
        .get("hooks", {})
        .get("vcs", {})
        .get("version-file", None)
    )
    if hatch_vcs is not None:
        yield hatch_vcs

    # [tool.pdm.version]
    # write_to = "_version.py"
    pdm_version = (
        pyproject.get("tool", {})
        .get("pdm", {})
        .get("version", {})
        .get("write_to", None)
    )
    if pdm_version is not None:
        yield pdm_version

    # [[tool.scikit-build.generate]]
    # path = "_version.py"
    for skbg in pyproject.get("tool", {}).get("scikit-build", {}).get("generate", []):
        scikit_build_generate = skbg.get("path", None)
        if scikit_build_generate is not None:
            yield scikit_build_generate


if __name__ == "__main__":
    print(*sorted(sdist_files(Path.cwd(), isolated=True, installer="pip")), sep="\n")



from __future__ import annotations

from pathlib import Path
from typing import Any

import pathspec

__all__ = ["backend_ignored_patterns"]


def __dir__() -> list[str]:
    return __all__


def glob_filter(patterns: list[str], files: frozenset[str]) -> frozenset[str]:
    """
    Filter out files based on glob patterns.
    """
    new_files = set(files)
    for pattern in patterns:
        results = {str(p.as_posix()) for p in Path().glob(pattern)}
        new_files -= results
    return frozenset(new_files)


def pathspec_filter(patterns: list[str], files: frozenset[str]) -> frozenset[str]:
    """
    Filter out files based on gitignore-style patterns.
    """
    spec = pathspec.GitIgnoreSpec.from_lines(patterns)
    return frozenset(p for p in files if not spec.match_file(p))


def backend_ignored_patterns(
    backend: str, pyproject: dict[str, Any], files: frozenset[str]
) -> frozenset[str]:
    """
    Return the ignored patterns for the given backend. If the generator is
    "none", then no patterns are ignored. If the generator is "auto", then the
    value is read from the pyproject.toml file. Any recognized backend can also
    be specified directly.
    """

    if backend == "none":
        return files

    backend_resolved = (
        pyproject.get("build-system", {}).get(
            "build-backend", "setuptools.build_meta.__legacy__"
        )
        if backend == "auto"
        else backend
    )

    if backend_resolved == "flit_core.buildapi":
        exclude = (
            pyproject.get("tool", {})
            .get("flit", {})
            .get("sdist", {})
            .get("exclude", [])
        )
        return glob_filter(exclude, files)
    if backend_resolved == "hatchling.build":
        exclude = (
            pyproject.get("tool", {})
            .get("hatch", {})
            .get("build", {})
            .get("targets", {})
            .get("sdist", {})
            .get("exclude", [])
        )
        return pathspec_filter(exclude, files)
    if backend_resolved == "scikit_build_core.build":
        exclude = (
            pyproject.get("tool", {})
            .get("scikit-build", {})
            .get("sdist", {})
            .get("exclude", [])
        )
        return pathspec_filter(exclude, files)
    if backend_resolved == "pdm.backend":
        exclude = (
            pyproject.get("tool", {})
            .get("pdm", {})
            .get("build", {})
            .get("excludes", [])
        )
        return glob_filter(exclude, files)
    if backend_resolved == "poetry.core.masonry.api":
        exclude = [
            x if isinstance(x, str) else x["path"]
            for x in pyproject.get("tool", {}).get("poetry", {}).get("exclude", [])
            if isinstance(x, str) or "sdist" in x.get("format", ["sdist"])
        ]
        return glob_filter(exclude, files)
    if backend_resolved == "maturin":
        exclude = pyproject.get("tool", {}).get("maturin", {}).get("exclude", [])
        return glob_filter(exclude, files)
    if backend != "auto":
        msg = f"Unknown backend: {backend} - please add support in check_dist.backends"
        raise ValueError(msg)
    return files



from __future__ import annotations

from check_sdist._compat import tomllib
from check_sdist.sdist import default_sdist_ignore


def test_default_sdist_ignore_setuptools_scm_version_file() -> None:
    pyproject_str = """
    [tool.setuptools_scm]
    version_file = "src/example/_version.py"
    """
    pyproject = tomllib.loads(pyproject_str)

    ignore = set(default_sdist_ignore(pyproject))

    assert "src/example/_version.py" in ignore


def test_default_sdist_ignore_hatch_vcs_version_file() -> None:
    pyproject_str = """
    [tool.hatch.build.hooks.vcs]
    version-file = "src/example/_version.py"
    """
    pyproject = tomllib.loads(pyproject_str)

    ignore = set(default_sdist_ignore(pyproject))

    assert "src/example/_version.py" in ignore


def test_default_sdist_ignore_pdm_version_write_to() -> None:
    pyproject_str = """
    [tool.pdm.version]
    write_to = "foo/_version.py"
    """
    pyproject = tomllib.loads(pyproject_str)

    ignore = set(default_sdist_ignore(pyproject))

    assert "foo/_version.py" in ignore


def test_default_sdist_ignore_scikit_build_generate_paths() -> None:
    pyproject_str = """
    [[tool.scikit-build.generate]]
    path = "src/example/_version.py"

    [[tool.scikit-build.generate]]
    template = "unused-no-path"

    [[tool.scikit-build.generate]]
    path = "python/pkg/version.py"
    """
    pyproject = tomllib.loads(pyproject_str)

    ignore = set(default_sdist_ignore(pyproject))

    assert "src/example/_version.py" in ignore
    assert "python/pkg/version.py" in ignore


def test_default_sdist_ignore_autogen_with_setuptools_defaults() -> None:
    pyproject_str = """
    [build-system]
    build-backend = "setuptools.build_meta"

    [tool.setuptools_scm]
    version_file = "src/example/_version.py"
    """
    pyproject = tomllib.loads(pyproject_str)

    ignore = set(default_sdist_ignore(pyproject))

    assert "*.dist-info" in ignore
    assert "*.egg-info" in ignore
    assert "setup.cfg" in ignore
    assert "src/example/_version.py" in ignore



from __future__ import annotations

import pytest

from check_sdist import __version__
from check_sdist.__main__ import main


def test_version(capsys: pytest.CaptureFixture[str]):
    with pytest.raises(SystemExit):
        main(["--version"])

    out, err = capsys.readouterr()
    assert out == f"check-sdist {__version__}\n"
    assert not err



#!/usr/bin/env -S uv run --script

# /// script
# dependencies = ["nox>=2025.02.09"]
# ///

from __future__ import annotations

from pathlib import Path

import nox

nox.needs_version = ">=2025.2.9"
nox.options.default_venv_backend = "uv|virtualenv"

PYPROJECT = nox.project.load_toml()


@nox.session
def lint(session: nox.Session) -> None:
    """
    Run the linter.
    """
    session.install("prek")
    session.run("prek", "run", "--all-files", *session.posargs)


@nox.session
def pylint(session: nox.Session) -> None:
    """
    Run PyLint.
    """
    # This needs to be installed into the package environment, and is slower
    # than a pre-commit check
    session.install("-e.", "pylint")
    session.run("pylint", "src", *session.posargs)


@nox.session
def tests(session: nox.Session) -> None:
    """
    Run the unit and regular tests.
    """
    test_grp = nox.project.dependency_groups(PYPROJECT, "test")
    session.install("-e.", *test_grp)
    session.run("pytest", "-nauto", *session.posargs, env={"COVERAGE_CORE": "sysmon"})


@nox.session(default=False)
def coverage(session: nox.Session) -> None:
    """
    Run tests and compute coverage.
    """
    session.posargs.extend(
        [
            "--cov",
            "--cov-config=pyproject.toml",
            "--cov-report=xml",
            "--cov-report=term-missing",
        ]
    )
    tests(session)


@nox.session(default=False)
def build(session: nox.Session) -> None:
    """
    Build an SDist and wheel.
    """
    session.install("build")
    session.run("python", "-m", "build")


@nox.session(tags=["gen"])
def generate_schema(session: nox.Session) -> None:
    """
    Generate a schema file.
    """
    deps = nox.project.load_toml("scripts/generate_schema.py")["dependencies"]
    session.install(*deps)
    out = session.run("python", "scripts/generate_schema.py", silent=True)
    path = Path("src/check_sdist/resources/check-sdist.schema.json")
    path.write_text(out)


@nox.session(default=False)
def bump(session: nox.Session) -> None:
    """
    Bump the version.
    """

    session.install("bump-my-version")
    session.run("bump-my-version", *session.posargs)


if __name__ == "__main__":
    nox.main()



from __future__ import annotations

import subprocess
from pathlib import Path


def git_files(source_dir: Path, *, recurse_submodules: bool = True) -> frozenset[str]:
    """Return the files that are tracked by git in the source directory."""

    cmd = ["git", "ls-files", "--cached"]
    if recurse_submodules:
        cmd.append("--recurse-submodules")
    return frozenset(
        subprocess.run(
            cmd,
            cwd=source_dir,
            text=True,
            capture_output=True,
            check=True,
        ).stdout.splitlines()
    )


if __name__ == "__main__":
    print(*sorted(git_files(Path.cwd())), sep="\n")
